"""Focomy - The Most Beautiful CMS."""

import secrets
import time
import uuid
from contextlib import asynccontextmanager

from fastapi import FastAPI, HTTPException, Request
from fastapi.middleware.cors import CORSMiddleware
from fastapi.middleware.gzip import GZipMiddleware
from fastapi.staticfiles import StaticFiles
from slowapi import _rate_limit_exceeded_handler
from slowapi.errors import RateLimitExceeded
from starlette.middleware.base import BaseHTTPMiddleware
from starlette.middleware.sessions import SessionMiddleware
from starlette.responses import Response

from .config import settings
from .database import close_db, init_db
from .rate_limit import limiter
from .services.logging import bind_context, clear_context, configure_logging, get_logger


class RedirectMiddleware(BaseHTTPMiddleware):
    """Handle URL redirects before other processing."""

    # Skip paths that should not be redirected
    SKIP_PREFIXES = ("/api/", "/admin/", "/static/", "/uploads/", "/health")

    async def dispatch(self, request: Request, call_next):
        path = request.url.path

        # Skip certain paths
        if any(path.startswith(prefix) for prefix in self.SKIP_PREFIXES):
            return await call_next(request)

        # Only check redirects for GET requests
        if request.method != "GET":
            return await call_next(request)

        try:
            from .database import async_session
            from .services.redirect import RedirectService

            async with async_session() as db:
                redirect_svc = RedirectService(db)
                query_string = str(request.url.query) if request.url.query else ""
                redirect = await redirect_svc.find_redirect(path, query_string)

                if redirect:
                    from starlette.responses import RedirectResponse

                    return RedirectResponse(
                        url=redirect["to_path"],
                        status_code=redirect["status_code"],
                    )
        except Exception:
            # Don't break the site if redirect service fails
            pass

        return await call_next(request)


class SecurityHeadersMiddleware(BaseHTTPMiddleware):
    """Add comprehensive security headers to all responses."""

    # CSP for admin pages (allow inline styles/scripts for Editor.js)
    ADMIN_CSP = (
        "default-src 'self'; "
        "script-src 'self' 'unsafe-inline' 'unsafe-eval' https://cdn.jsdelivr.net https://unpkg.com; "
        "style-src 'self' 'unsafe-inline'; "
        "img-src 'self' data: https: blob:; "
        "font-src 'self' https://fonts.gstatic.com; "
        "frame-src https://www.youtube.com https://player.vimeo.com https://open.spotify.com "
        "https://w.soundcloud.com https://www.google.com https://maps.google.com; "
        "connect-src 'self' https://www.googletagmanager.com"
    )

    # CSP for public pages (more restrictive)
    PUBLIC_CSP = (
        "default-src 'self'; "
        "script-src 'self' 'unsafe-inline' https://www.googletagmanager.com https://www.google-analytics.com; "
        "style-src 'self' 'unsafe-inline'; "
        "img-src 'self' data: https: blob:; "
        "font-src 'self' https://fonts.gstatic.com; "
        "frame-src https://www.youtube.com https://player.vimeo.com https://open.spotify.com "
        "https://w.soundcloud.com https://www.google.com https://maps.google.com; "
        "connect-src 'self' https://www.google-analytics.com; "
        "object-src 'none'; "
        "base-uri 'self'; "
        "form-action 'self'; "
        "upgrade-insecure-requests"
    )

    # Permissions Policy (formerly Feature-Policy)
    PERMISSIONS_POLICY = (
        "accelerometer=(), "
        "ambient-light-sensor=(), "
        "autoplay=(self), "
        "battery=(), "
        "camera=(), "
        "display-capture=(), "
        "document-domain=(), "
        "encrypted-media=(self), "
        "fullscreen=(self), "
        "geolocation=(), "
        "gyroscope=(), "
        "magnetometer=(), "
        "microphone=(), "
        "midi=(), "
        "payment=(), "
        "picture-in-picture=(self), "
        "publickey-credentials-get=(), "
        "screen-wake-lock=(), "
        "usb=(), "
        "xr-spatial-tracking=()"
    )

    async def dispatch(self, request: Request, call_next):
        response = await call_next(request)
        headers_config = settings.security.headers

        # Basic security headers (always applied)
        response.headers["X-Content-Type-Options"] = "nosniff"
        response.headers["X-XSS-Protection"] = "1; mode=block"
        response.headers["Referrer-Policy"] = "strict-origin-when-cross-origin"

        # X-Frame-Options
        if headers_config.frame_options:
            response.headers["X-Frame-Options"] = headers_config.frame_options

        # HSTS (only for HTTPS in production)
        if headers_config.hsts_enabled and not settings.debug:
            hsts_value = f"max-age={headers_config.hsts_max_age}"
            if headers_config.hsts_include_subdomains:
                hsts_value += "; includeSubDomains"
            if headers_config.hsts_preload:
                hsts_value += "; preload"
            response.headers["Strict-Transport-Security"] = hsts_value

        # Content Security Policy
        if headers_config.csp_enabled:
            csp_header = "Content-Security-Policy"
            if headers_config.csp_report_only:
                csp_header = "Content-Security-Policy-Report-Only"

            if request.url.path.startswith("/admin"):
                response.headers[csp_header] = self.ADMIN_CSP
            elif not request.url.path.startswith("/api"):
                response.headers[csp_header] = self.PUBLIC_CSP

        # Permissions Policy
        if headers_config.permissions_policy_enabled:
            response.headers["Permissions-Policy"] = self.PERMISSIONS_POLICY

        # Cross-Origin headers for additional isolation
        response.headers["Cross-Origin-Opener-Policy"] = "same-origin"
        response.headers["Cross-Origin-Resource-Policy"] = "same-origin"

        return response


class CSRFMiddleware(BaseHTTPMiddleware):
    """CSRF protection middleware.

    This middleware:
    1. Generates/retrieves CSRF tokens and stores in cookies
    2. Makes token available in request.state for templates
    3. Validates X-CSRF-Token header for AJAX requests

    For form submissions, routes should validate CSRF using the
    validate_csrf_token() helper function.
    """

    SAFE_METHODS = {"GET", "HEAD", "OPTIONS", "TRACE"}
    CSRF_COOKIE_NAME = "csrf_token"
    CSRF_HEADER_NAME = "x-csrf-token"

    async def dispatch(self, request: Request, call_next):
        # Skip CSRF for API routes (use bearer tokens)
        if request.url.path.startswith("/api/"):
            return await call_next(request)

        # Get or create CSRF token
        csrf_token = request.cookies.get(self.CSRF_COOKIE_NAME)
        if not csrf_token:
            csrf_token = secrets.token_urlsafe(32)

        # Store token in request state for templates
        request.state.csrf_token = csrf_token

        # Validate CSRF header for unsafe methods (AJAX requests)
        if request.method not in self.SAFE_METHODS:
            header_token = request.headers.get(self.CSRF_HEADER_NAME)
            content_type = request.headers.get("content-type", "")

            # For AJAX requests with JSON, require header
            if "application/json" in content_type:
                if header_token != csrf_token:
                    raise HTTPException(status_code=403, detail="CSRF token required")
            # For form submissions, let the route handler validate via form field
            # (This avoids body consumption issues in middleware)

        response = await call_next(request)

        # Set CSRF cookie
        response.set_cookie(
            key=self.CSRF_COOKIE_NAME,
            value=csrf_token,
            httponly=False,  # Needs to be readable by JS for AJAX
            samesite="lax",
            secure=not settings.debug,
        )

        return response


class RequestLoggingMiddleware(BaseHTTPMiddleware):
    """Structured request/response logging middleware.

    Logs every request with timing, status, and context.
    Assigns a unique request ID for tracing.
    """

    # Skip logging for these paths to reduce noise
    SKIP_PATHS = {"/health", "/favicon.ico"}

    async def dispatch(self, request: Request, call_next):
        # Skip health checks and favicon
        if request.url.path in self.SKIP_PATHS:
            return await call_next(request)

        # Generate request ID
        request_id = str(uuid.uuid4())[:8]
        request.state.request_id = request_id

        # Bind context for all logs in this request
        bind_context(
            request_id=request_id,
            method=request.method,
            path=request.url.path,
        )

        # Get client IP (handle proxies)
        client_ip = request.headers.get("x-forwarded-for", "").split(",")[0].strip()
        if not client_ip:
            client_ip = request.client.host if request.client else "unknown"

        logger = get_logger("focomy.http")
        start_time = time.perf_counter()

        try:
            response = await call_next(request)
            duration_ms = (time.perf_counter() - start_time) * 1000

            # Log based on status code
            log_data = {
                "client_ip": client_ip,
                "status": response.status_code,
                "duration_ms": round(duration_ms, 2),
                "user_agent": request.headers.get("user-agent", "")[:100],
            }

            if response.status_code >= 500:
                logger.error("Request failed", **log_data)
            elif response.status_code >= 400:
                logger.warning("Request error", **log_data)
            else:
                logger.info("Request completed", **log_data)

            # Add request ID to response headers for debugging
            response.headers["X-Request-ID"] = request_id

            return response

        except Exception as e:
            duration_ms = (time.perf_counter() - start_time) * 1000
            logger.exception(
                "Request exception",
                client_ip=client_ip,
                duration_ms=round(duration_ms, 2),
                error=str(e),
            )
            raise
        finally:
            clear_context()


def validate_csrf_token(request: Request, form_token: str) -> bool:
    """Validate CSRF token from form data against cookie.

    Call this in form-handling routes:
        if not validate_csrf_token(request, form.csrf_token):
            raise HTTPException(403, "CSRF token mismatch")
    """
    cookie_token = request.cookies.get("csrf_token")
    return cookie_token and form_token == cookie_token


@asynccontextmanager
async def lifespan(app: FastAPI):
    """Application lifespan handler."""
    # Configure structured logging first
    configure_logging()
    logger = get_logger("focomy")
    logger.info("Starting Focomy CMS", version="0.1.0", debug=settings.debug)

    # Startup
    await init_db()
    logger.info("Database initialized")

    # Security warnings
    if settings.security.secret_key == "change-this-in-production":
        logger.warning(
            "SECURITY: secret_key is using default value. "
            "Set FOCOMY_SECURITY__SECRET_KEY environment variable in production."
        )
    if "*" in settings.cors.allow_origins:
        logger.warning(
            "SECURITY: CORS allows all origins (*). "
            "Set FOCOMY_CORS__ALLOW_ORIGINS to specific domains in production."
        )

    # Configure OAuth
    from .services.oauth import oauth_service

    oauth_service.configure(app)
    logger.info("OAuth configured")

    yield

    # Shutdown
    logger.info("Shutting down Focomy CMS")
    await close_db()


app = FastAPI(
    title="Focomy",
    description="The Most Beautiful CMS",
    version="0.1.0",
    lifespan=lifespan,
)

# Rate limiting
app.state.limiter = limiter
app.add_exception_handler(RateLimitExceeded, _rate_limit_exceeded_handler)

# Security middlewares (order matters - first added = last executed)
app.add_middleware(GZipMiddleware, minimum_size=1000)
app.add_middleware(SecurityHeadersMiddleware)
app.add_middleware(CSRFMiddleware)
app.add_middleware(SessionMiddleware, secret_key=settings.security.secret_key)
app.add_middleware(RedirectMiddleware)

# Request logging (runs early, logs all requests)
app.add_middleware(RequestLoggingMiddleware)

# CORS middleware (configurable via settings)
if settings.cors.enabled:
    app.add_middleware(
        CORSMiddleware,
        allow_origins=settings.cors.allow_origins,
        allow_credentials=settings.cors.allow_credentials,
        allow_methods=settings.cors.allow_methods,
        allow_headers=settings.cors.allow_headers,
        max_age=settings.cors.max_age,
    )

# Static files - ディレクトリが存在する場合のみマウント
uploads_dir = settings.base_dir / "uploads"
uploads_dir.mkdir(exist_ok=True)
app.mount(
    "/uploads",
    StaticFiles(directory=str(uploads_dir)),
    name="uploads",
)

# Static assets (favicon, css, js)
static_dir = settings.base_dir / "static"
static_dir.mkdir(exist_ok=True)
app.mount(
    "/static",
    StaticFiles(directory=str(static_dir)),
    name="static",
)


@app.get("/health")
async def health():
    """Health check endpoint."""
    return {"status": "healthy"}


# Exception handlers
from fastapi.responses import HTMLResponse
from starlette.exceptions import HTTPException as StarletteHTTPException


@app.exception_handler(404)
async def not_found_handler(request: Request, exc: StarletteHTTPException):
    """Handle 404 errors."""
    # Return JSON for API routes
    if request.url.path.startswith("/api/"):
        return Response(
            content='{"detail": "Not found"}', status_code=404, media_type="application/json"
        )

    # Render HTML error page for frontend
    from .services.theme import theme_service

    try:
        html = theme_service.render(
            "404.html",
            {"site_name": "Focomy"},
        )
        return HTMLResponse(content=html, status_code=404)
    except Exception:
        return HTMLResponse(content="<h1>404 - Not Found</h1>", status_code=404)


@app.exception_handler(500)
async def server_error_handler(request: Request, exc: Exception):
    """Handle 500 errors."""
    # Return JSON for API routes
    if request.url.path.startswith("/api/"):
        return Response(
            content='{"detail": "Internal server error"}',
            status_code=500,
            media_type="application/json",
        )

    # Render HTML error page for frontend
    from .services.theme import theme_service

    try:
        html = theme_service.render(
            "500.html",
            {"site_name": "Focomy"},
        )
        return HTMLResponse(content=html, status_code=500)
    except Exception:
        return HTMLResponse(content="<h1>500 - Internal Server Error</h1>", status_code=500)


# Include routers
from .admin import routes as admin
from .api import auth, comments, entities, forms, media, relations, revisions, schema, search, seo
from .engine import routes as engine
from .utils import is_feature_enabled

# Phase 1: Core APIs (always enabled)
app.include_router(entities.router, prefix="/api")
app.include_router(schema.router, prefix="/api")
app.include_router(relations.router, prefix="/api")
app.include_router(auth.router, prefix="/api")
app.include_router(seo.router)
app.include_router(admin.router)

# Phase 2: Media
if is_feature_enabled("media"):
    app.include_router(media.router, prefix="/api")

# Phase 4: Search, Revisions
if is_feature_enabled("search"):
    app.include_router(search.router, prefix="/api")
if is_feature_enabled("revision"):
    app.include_router(revisions.router, prefix="/api")

# Phase 5: Comments, Forms
if is_feature_enabled("comment"):
    app.include_router(comments.router, prefix="/api")
if is_feature_enabled("form"):
    app.include_router(forms.router)


@app.get("/api/health")
async def api_health():
    """Health check endpoint for monitoring."""
    return {"status": "healthy"}


# Engine router must be last (has catch-all routes)
app.include_router(engine.router)



"""WordPress Import Service - Orchestrates full import with EntityService integration."""

from __future__ import annotations

import asyncio
import logging
from dataclasses import asdict
from datetime import datetime, timezone

from ...utils import utcnow
from pathlib import Path
from typing import Any

from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession

from ...models import Entity, ImportJob, ImportJobPhase, ImportJobStatus
from ..entity import EntityService
from ..field import field_service
from .analyzer import WordPressAnalyzer
from .content_sanitizer import ContentSanitizer, SanitizeResult
from .constants import WP_STATUS_MAP
from .importer import ImportConfig, ImportProgress, ImportResult, WordPressImporter
from .link_fixer import InternalLinkFixer, URLMapBuilder
from .media import MediaImporter
from .redirects import RedirectGenerator, RedirectReport
from .rest_client import RESTClientConfig, WordPressRESTClient
from .wxr_parser import WXRData, WXRParser

logger = logging.getLogger(__name__)


class WordPressImportService:
    """
    Full WordPress import service with database integration.

    Handles:
    - Job creation and tracking
    - WXR file or REST API as source
    - Progress updates
    - EntityService integration for actual data import
    """

    def __init__(self, db: AsyncSession):
        self.db = db
        self.entity_svc = EntityService(db)
        self.sanitizer = ContentSanitizer()

    async def create_job(
        self,
        source_type: str,
        source_url: str | None = None,
        source_file: str | None = None,
        config: dict | None = None,
        user_id: str | None = None,
    ) -> ImportJob:
        """Create a new import job."""
        job = ImportJob(
            source_type=source_type,
            source_url=source_url,
            source_file=source_file,
            config=config or {},
            created_by=user_id,
            status=ImportJobStatus.PENDING,
            phase=ImportJobPhase.INIT,
        )
        self.db.add(job)
        await self.db.commit()
        await self.db.refresh(job)
        return job

    async def get_job(self, job_id: str) -> ImportJob | None:
        """Get import job by ID."""
        result = await self.db.execute(select(ImportJob).where(ImportJob.id == job_id))
        return result.scalar_one_or_none()

    async def get_active_jobs(self) -> list[ImportJob]:
        """Get all active (non-completed) jobs."""
        result = await self.db.execute(
            select(ImportJob).where(
                ImportJob.status.in_([
                    ImportJobStatus.PENDING,
                    ImportJobStatus.ANALYZING,
                    ImportJobStatus.IMPORTING,
                ])
            )
        )
        return list(result.scalars().all())

    async def update_job(
        self,
        job_id: str,
        status: str | None = None,
        phase: str | None = None,
        progress_current: int | None = None,
        progress_total: int | None = None,
        progress_message: str | None = None,
        **kwargs,
    ) -> ImportJob | None:
        """Update import job status and progress."""
        job = await self.get_job(job_id)
        if not job:
            return None

        if status:
            job.status = status
        if phase:
            job.phase = phase
        if progress_current is not None:
            job.progress_current = progress_current
        if progress_total is not None:
            job.progress_total = progress_total
        if progress_message is not None:
            job.progress_message = progress_message

        for key, value in kwargs.items():
            if hasattr(job, key):
                setattr(job, key, value)

        await self.db.commit()
        await self.db.refresh(job)
        return job

    async def analyze(self, job_id: str) -> dict | None:
        """
        Analyze source without importing.

        Updates job with analysis results.
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        try:
            await self.update_job(
                job_id,
                status=ImportJobStatus.ANALYZING,
                phase=ImportJobPhase.ANALYZE,
                progress_message="Analyzing WordPress data...",
            )

            # Get WXR data from source
            wxr_data = await self._fetch_source_data(job)
            if not wxr_data:
                raise ValueError("Failed to fetch source data")

            # Analyze
            analyzer = WordPressAnalyzer()
            report = analyzer.analyze_data(wxr_data)

            # Convert report to dict
            analysis_dict = {
                "site_url": report.site_url,
                "site_name": report.site_name,
                "wp_version": report.wp_version,
                "language": report.language,
                "posts": {
                    "total": report.posts.total,
                    "published": report.posts.published,
                    "draft": report.posts.draft,
                },
                "pages": {
                    "total": report.pages.total,
                    "published": report.pages.published,
                    "draft": report.pages.draft,
                },
                "media": {
                    "total_count": report.media_stats.total_count,
                    "by_type": report.media_stats.by_type,
                },
                "categories_count": len(report.categories),
                "tags_count": len(report.tags),
                "users_count": report.users_count,
                "comments_count": report.comments_count,
                "menus_count": report.menus_count,
                "custom_post_types": [
                    {"name": cpt.name, "count": cpt.count}
                    for cpt in report.custom_post_types
                ],
                "detected_plugins": [
                    {"name": p.name, "slug": p.slug}
                    for p in report.detected_plugins
                ],
                "warnings": [
                    {"code": w.code, "message": w.message}
                    for w in report.warnings
                ],
                "recommendations": report.recommendations,
                "estimated_time": report.estimated_time,
                "estimated_storage": report.estimated_storage,
            }

            await self.update_job(
                job_id,
                status=ImportJobStatus.PENDING,
                phase=ImportJobPhase.INIT,
                analysis=analysis_dict,
                progress_message="Analysis complete",
            )

            return analysis_dict

        except Exception as e:
            logger.exception(f"Analysis failed for job {job_id}")
            await self.update_job(
                job_id,
                status=ImportJobStatus.FAILED,
                errors=[str(e)],
                progress_message=f"Analysis failed: {str(e)}",
            )
            return None

    async def run_import(
        self,
        job_id: str,
        resume: bool = False,
    ) -> ImportResult | None:
        """
        Run full import.

        This is the main import method that should be called from background task.

        Args:
            job_id: The import job ID
            resume: If True, skip already processed items (from checkpoint)
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        try:
            if resume:
                checkpoint = await self._get_checkpoint(job_id)
                last_phase = checkpoint.get("last_phase")
                await self.update_job(
                    job_id,
                    status=ImportJobStatus.IMPORTING,
                    progress_message=f"Resuming import from {last_phase or 'beginning'}...",
                )
            else:
                await self.update_job(
                    job_id,
                    status=ImportJobStatus.IMPORTING,
                    phase=ImportJobPhase.CONNECT,
                    started_at=utcnow(),
                    progress_message="Starting import...",
                )

            # Get WXR data from source
            wxr_data = await self._fetch_source_data(job)
            if not wxr_data:
                raise ValueError("Failed to fetch source data")

            # Import each type
            result = ImportResult()

            # Authors
            await self.update_job(
                job_id,
                phase=ImportJobPhase.AUTHORS,
                progress_current=0,
                progress_total=len(wxr_data.authors),
                progress_message="Importing authors...",
            )
            authors_count = await self._import_authors(job_id, wxr_data, resume=resume)
            result.authors_imported = authors_count

            # Categories
            await self.update_job(
                job_id,
                phase=ImportJobPhase.CATEGORIES,
                progress_current=0,
                progress_total=len(wxr_data.categories),
                progress_message="Importing categories...",
            )
            cats_count = await self._import_categories(job_id, wxr_data, resume=resume)
            result.categories_imported = cats_count

            # Tags
            await self.update_job(
                job_id,
                phase=ImportJobPhase.TAGS,
                progress_current=0,
                progress_total=len(wxr_data.tags),
                progress_message="Importing tags...",
            )
            tags_count = await self._import_tags(job_id, wxr_data, resume=resume)
            result.tags_imported = tags_count

            # Media
            config = job.config or {}
            if config.get("import_media", True):
                media_posts = [p for p in wxr_data.posts if p.post_type == "attachment"]
                await self.update_job(
                    job_id,
                    phase=ImportJobPhase.MEDIA,
                    progress_current=0,
                    progress_total=len(media_posts),
                    progress_message="Importing media...",
                )
                media_count = await self._import_media(job_id, wxr_data, config, resume=resume)
                result.media_imported = media_count

            # Posts
            posts = [p for p in wxr_data.posts if p.post_type == "post"]
            await self.update_job(
                job_id,
                phase=ImportJobPhase.POSTS,
                progress_current=0,
                progress_total=len(posts),
                progress_message="Importing posts...",
            )
            posts_count = await self._import_posts(job_id, wxr_data, "post", resume=resume)
            result.posts_imported = posts_count

            # Pages
            pages = [p for p in wxr_data.posts if p.post_type == "page"]
            await self.update_job(
                job_id,
                phase=ImportJobPhase.PAGES,
                progress_current=0,
                progress_total=len(pages),
                progress_message="Importing pages...",
            )
            pages_count = await self._import_posts(job_id, wxr_data, "page", resume=resume)
            result.pages_imported = pages_count

            # Menus
            if config.get("import_menus", True) and wxr_data.menus:
                await self.update_job(
                    job_id,
                    phase=ImportJobPhase.MENUS,
                    progress_current=0,
                    progress_total=len(wxr_data.menus),
                    progress_message="Importing menus...",
                )
                menus_count = await self._import_menus(job_id, wxr_data, resume=resume)
                result.menus_imported = menus_count

            # Complete
            result.success = True
            await self.update_job(
                job_id,
                status=ImportJobStatus.COMPLETED,
                phase=ImportJobPhase.COMPLETE,
                completed_at=utcnow(),
                posts_imported=result.posts_imported,
                pages_imported=result.pages_imported,
                media_imported=result.media_imported,
                categories_imported=result.categories_imported,
                tags_imported=result.tags_imported,
                authors_imported=result.authors_imported,
                menus_imported=result.menus_imported,
                progress_message="Import completed successfully!",
            )

            return result

        except Exception as e:
            logger.exception(f"Import failed for job {job_id}")
            await self.update_job(
                job_id,
                status=ImportJobStatus.FAILED,
                completed_at=utcnow(),
                errors=[str(e)],
                progress_message=f"Import failed: {str(e)}",
            )
            return None

    async def _fetch_source_data(self, job: ImportJob) -> WXRData | None:
        """Fetch WXR data from source."""
        if job.source_type == "wxr":
            if not job.source_file:
                return None
            parser = WXRParser()
            return parser.parse(Path(job.source_file))

        elif job.source_type == "rest":
            if not job.source_url:
                return None

            config = job.config or {}
            rest_config = RESTClientConfig(
                site_url=job.source_url,
                username=config.get("username", ""),
                password=config.get("password", ""),
            )

            async with WordPressRESTClient(rest_config) as client:
                return await client.fetch_all(
                    include_drafts=config.get("include_drafts", True),
                    include_media=config.get("import_media", True),
                    include_comments=config.get("import_comments", True),
                    include_menus=config.get("import_menus", True),
                )

        return None

    async def _import_authors(
        self,
        job_id: str,
        wxr_data: WXRData,
        resume: bool = False,
    ) -> int:
        """Import authors as users."""
        count = 0
        skipped = 0
        for i, author in enumerate(wxr_data.authors):
            try:
                # Skip if already processed in previous run
                if resume and await self._is_processed(job_id, "authors", author.id):
                    skipped += 1
                    continue

                # Check if user already exists
                existing = await self._find_by_wp_id("user", author.id)
                if existing:
                    # Save checkpoint even for duplicates
                    await self._save_checkpoint(job_id, "authors", author.id, "authors")
                    continue

                # Create user entity
                user_data = {
                    "name": author.display_name or author.login,
                    "email": author.email or f"{author.login}@imported.local",
                    "role": "author",
                    "wp_id": author.id,
                    "wp_login": author.login,
                }

                await self.entity_svc.create("user", user_data)
                count += 1

                # Save checkpoint after successful import
                await self._save_checkpoint(job_id, "authors", author.id, "authors")

                await self.update_job(
                    job_id,
                    progress_current=i + 1,
                    progress_message=f"Imported author: {author.display_name}",
                )

            except Exception as e:
                logger.warning(f"Failed to import author {author.login}: {e}")

        await self.update_job(job_id, authors_imported=count)
        if skipped:
            logger.info(f"Skipped {skipped} already processed authors")
        return count

    async def _import_categories(
        self,
        job_id: str,
        wxr_data: WXRData,
        resume: bool = False,
    ) -> int:
        """Import categories."""
        count = 0
        skipped = 0

        # Sort by parent to import parents first
        categories = sorted(wxr_data.categories, key=lambda c: c.parent_id)

        for i, cat in enumerate(categories):
            try:
                # Skip if already processed in previous run
                if resume and await self._is_processed(job_id, "categories", cat.id):
                    skipped += 1
                    continue

                existing = await self._find_by_wp_id("category", cat.id)
                if existing:
                    await self._save_checkpoint(job_id, "categories", cat.id, "categories")
                    continue

                cat_data = {
                    "name": cat.name,
                    "slug": cat.slug,
                    "description": cat.description,
                    "wp_id": cat.id,
                    "wp_parent_id": cat.parent_id,
                }

                await self.entity_svc.create("category", cat_data)
                count += 1

                # Save checkpoint after successful import
                await self._save_checkpoint(job_id, "categories", cat.id, "categories")

                await self.update_job(
                    job_id,
                    progress_current=i + 1,
                    progress_message=f"Imported category: {cat.name}",
                )

            except Exception as e:
                logger.warning(f"Failed to import category {cat.name}: {e}")

        await self.update_job(job_id, categories_imported=count)
        if skipped:
            logger.info(f"Skipped {skipped} already processed categories")
        return count

    async def _import_tags(
        self,
        job_id: str,
        wxr_data: WXRData,
        resume: bool = False,
    ) -> int:
        """Import tags."""
        count = 0
        skipped = 0
        for i, tag in enumerate(wxr_data.tags):
            try:
                # Skip if already processed in previous run
                if resume and await self._is_processed(job_id, "tags", tag.id):
                    skipped += 1
                    continue

                existing = await self._find_by_wp_id("tag", tag.id)
                if existing:
                    await self._save_checkpoint(job_id, "tags", tag.id, "tags")
                    continue

                tag_data = {
                    "name": tag.name,
                    "slug": tag.slug,
                    "description": tag.description,
                    "wp_id": tag.id,
                }

                await self.entity_svc.create("tag", tag_data)
                count += 1

                # Save checkpoint after successful import
                await self._save_checkpoint(job_id, "tags", tag.id, "tags")

                await self.update_job(
                    job_id,
                    progress_current=i + 1,
                    progress_message=f"Imported tag: {tag.name}",
                )

            except Exception as e:
                logger.warning(f"Failed to import tag {tag.name}: {e}")

        await self.update_job(job_id, tags_imported=count)
        if skipped:
            logger.info(f"Skipped {skipped} already processed tags")
        return count

    async def _import_media(
        self,
        job_id: str,
        wxr_data: WXRData,
        config: dict,
        resume: bool = False,
    ) -> int:
        """Import media files.

        If config['download_media'] is True, actually downloads the files.
        If config['convert_to_webp'] is True, converts images to WebP format.
        """
        count = 0
        skipped = 0
        media_posts = [p for p in wxr_data.posts if p.post_type == "attachment"]

        # Initialize MediaImporter if downloading files
        media_importer = None
        download_media = config.get("download_media", False)

        if download_media:
            from pathlib import Path

            upload_dir = Path(config.get("upload_dir", "uploads"))
            base_url = config.get("base_url", "")

            media_importer = MediaImporter(
                upload_dir=upload_dir,
                base_url=base_url,
                convert_to_webp=config.get("convert_to_webp", False),
                webp_quality=config.get("webp_quality", 85),
                max_image_size=config.get("max_image_size", 2048),
                jpeg_quality=config.get("jpeg_quality", 85),
            )

        for i, media in enumerate(media_posts):
            try:
                # Skip if already processed in previous run
                if resume and await self._is_processed(job_id, "media", media.id):
                    skipped += 1
                    continue

                existing = await self._find_by_wp_id("media", media.id)
                if existing:
                    await self._save_checkpoint(job_id, "media", media.id, "media")
                    continue

                # Base media data
                media_data = {
                    "title": media.title,
                    "slug": media.slug,
                    "alt_text": media.postmeta.get("_wp_attachment_image_alt", ""),
                    "caption": media.excerpt,
                    "description": media.content,
                    "source_url": media.guid,
                    "wp_id": media.id,
                }

                # Download and process file if enabled
                if media_importer and media.guid:
                    from .media import MediaItem

                    item = MediaItem(
                        original_url=media.guid,
                        filename=Path(media.guid).name if media.guid else f"media_{media.id}",
                        post_id=media.id,
                        title=media.title,
                        alt_text=media.postmeta.get("_wp_attachment_image_alt", ""),
                    )

                    result = await media_importer.import_media([item])
                    if result.imported:
                        imported = result.imported[0]
                        media_data["filename"] = imported.filename
                        media_data["stored_path"] = imported.new_path
                        media_data["mime_type"] = imported.mime_type
                        media_data["size"] = imported.file_size
                        media_data["width"] = imported.width
                        media_data["height"] = imported.height

                await self.entity_svc.create("media", media_data)
                count += 1

                # Save checkpoint after successful import
                await self._save_checkpoint(job_id, "media", media.id, "media")

                await self.update_job(
                    job_id,
                    progress_current=i + 1,
                    progress_message=f"Imported media: {media.title[:50] if media.title else 'untitled'}",
                )

            except Exception as e:
                logger.warning(f"Failed to import media {media.id}: {e}")

        await self.update_job(job_id, media_imported=count)
        if skipped:
            logger.info(f"Skipped {skipped} already processed media")
        return count

    async def _import_posts(
        self,
        job_id: str,
        wxr_data: WXRData,
        post_type: str,
        resume: bool = False,
    ) -> int:
        """Import posts or pages."""
        count = 0
        skipped = 0
        posts = [p for p in wxr_data.posts if p.post_type == post_type]

        # Checkpoint key: "posts" or "pages"
        checkpoint_key = "posts" if post_type == "post" else "pages"

        for i, post in enumerate(posts):
            try:
                # Skip if already processed in previous run
                if resume and await self._is_processed(job_id, checkpoint_key, post.id):
                    skipped += 1
                    continue

                existing = await self._find_by_wp_id(post_type, post.id)
                if existing:
                    await self._save_checkpoint(job_id, checkpoint_key, post.id, checkpoint_key)
                    continue

                # Sanitize content and excerpt
                content_result = self.sanitizer.sanitize(post.content or "")
                excerpt_result = self.sanitizer.sanitize(post.excerpt or "")

                # Log warnings if dangerous content was found
                if content_result.had_issues:
                    logger.warning(
                        f"Sanitized dangerous content in {post_type} {post.id}: "
                        f"{len(content_result.warnings)} issues"
                    )
                if excerpt_result.had_issues:
                    logger.warning(
                        f"Sanitized dangerous content in {post_type} {post.id} excerpt: "
                        f"{len(excerpt_result.warnings)} issues"
                    )

                post_data = {
                    "title": post.title,
                    "slug": post.slug,
                    "content": content_result.content,
                    "excerpt": excerpt_result.content,
                    "status": WP_STATUS_MAP.get(post.status, "draft"),
                    "wp_id": post.id,
                    "wp_author_id": post.author_id,
                    "created_at": post.created_at.isoformat(),
                    "updated_at": post.modified_at.isoformat(),
                }

                # SEO data from Yoast
                if "_yoast_wpseo_title" in post.postmeta:
                    post_data["seo_title"] = post.postmeta["_yoast_wpseo_title"]
                if "_yoast_wpseo_metadesc" in post.postmeta:
                    post_data["seo_description"] = post.postmeta["_yoast_wpseo_metadesc"]

                # Categories and tags as comma-separated slugs
                if post.categories:
                    post_data["category_slugs"] = ",".join(c["slug"] for c in post.categories)
                if post.tags:
                    post_data["tag_slugs"] = ",".join(t["slug"] for t in post.tags)

                await self.entity_svc.create(post_type, post_data)
                count += 1

                # Save checkpoint after successful import
                await self._save_checkpoint(job_id, checkpoint_key, post.id, checkpoint_key)

                await self.update_job(
                    job_id,
                    progress_current=i + 1,
                    progress_message=f"Imported {post_type}: {post.title[:50]}",
                )

            except Exception as e:
                logger.warning(f"Failed to import {post_type} {post.id}: {e}")

        if skipped:
            logger.info(f"Skipped {skipped} already processed {post_type}s")
        return count

    async def _import_menus(
        self,
        job_id: str,
        wxr_data: WXRData,
        resume: bool = False,
    ) -> int:
        """Import navigation menus."""
        count = 0
        skipped = 0
        for menu_name, items in wxr_data.menus.items():
            try:
                # Skip if already processed in previous run (use menu_name as ID)
                if resume and await self._is_menu_processed(job_id, menu_name):
                    skipped += 1
                    continue

                # Create menu entity
                menu_data = {
                    "name": menu_name,
                    "slug": menu_name.lower().replace(" ", "-"),
                    "items": [
                        {
                            "title": item.title,
                            "url": item.url,
                            "parent_id": item.parent_id,
                            "order": item.order,
                            "object_type": item.object_type,
                            "object_id": item.object_id,
                        }
                        for item in items
                    ],
                }

                await self.entity_svc.create("menu", menu_data)
                count += 1

                # Save checkpoint after successful import
                await self._save_menu_checkpoint(job_id, menu_name)

                await self.update_job(
                    job_id,
                    progress_current=count,
                    progress_message=f"Imported menu: {menu_name}",
                )

            except Exception as e:
                logger.warning(f"Failed to import menu {menu_name}: {e}")

        await self.update_job(job_id, menus_imported=count)
        if skipped:
            logger.info(f"Skipped {skipped} already processed menus")
        return count

    async def _is_menu_processed(self, job_id: str, menu_name: str) -> bool:
        """Check if a menu was already processed (uses name instead of ID)."""
        checkpoint = await self._get_checkpoint(job_id)
        return menu_name in checkpoint.get("menus", [])

    async def _save_menu_checkpoint(self, job_id: str, menu_name: str) -> None:
        """Save a processed menu to checkpoint (uses name instead of ID)."""
        job = await self.get_job(job_id)
        if not job:
            return

        checkpoint = job.checkpoint or {
            "authors": [],
            "categories": [],
            "tags": [],
            "media": [],
            "posts": [],
            "pages": [],
            "menus": [],
            "last_phase": None,
        }

        if "menus" not in checkpoint:
            checkpoint["menus"] = []

        if menu_name not in checkpoint["menus"]:
            checkpoint["menus"].append(menu_name)

        checkpoint["last_phase"] = "menus"

        await self.update_job(job_id, checkpoint=checkpoint)

    async def _find_by_wp_id(self, entity_type: str, wp_id: int) -> Entity | None:
        """Find entity by WordPress ID."""
        from ...models import EntityValue

        result = await self.db.execute(
            select(Entity)
            .join(EntityValue)
            .where(
                Entity.type == entity_type,
                EntityValue.field_name == "wp_id",
                EntityValue.value_int == wp_id,
                Entity.deleted_at.is_(None),
            )
        )
        return result.scalar_one_or_none()

    async def _get_checkpoint(self, job_id: str) -> dict:
        """Get checkpoint data for a job."""
        job = await self.get_job(job_id)
        if not job or not job.checkpoint:
            return {
                "authors": [],
                "categories": [],
                "tags": [],
                "media": [],
                "posts": [],
                "pages": [],
                "menus": [],
                "last_phase": None,
            }
        return job.checkpoint

    async def _save_checkpoint(
        self,
        job_id: str,
        item_type: str,
        wp_id: int,
        phase: str | None = None,
    ) -> None:
        """Save a processed item to checkpoint."""
        job = await self.get_job(job_id)
        if not job:
            return

        checkpoint = job.checkpoint or {
            "authors": [],
            "categories": [],
            "tags": [],
            "media": [],
            "posts": [],
            "pages": [],
            "menus": [],
            "last_phase": None,
        }

        if item_type in checkpoint and wp_id not in checkpoint[item_type]:
            checkpoint[item_type].append(wp_id)

        if phase:
            checkpoint["last_phase"] = phase

        await self.update_job(job_id, checkpoint=checkpoint)

    async def _is_processed(self, job_id: str, item_type: str, wp_id: int) -> bool:
        """Check if an item was already processed in a previous run."""
        checkpoint = await self._get_checkpoint(job_id)
        return wp_id in checkpoint.get(item_type, [])

    async def resume_import(self, job_id: str) -> dict | None:
        """
        Resume a failed or cancelled import from the last checkpoint.

        Returns the result of continuing the import.
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        if job.status not in (
            ImportJobStatus.FAILED,
            ImportJobStatus.CANCELLED,
        ):
            return {
                "success": False,
                "error": f"Cannot resume job with status: {job.status}",
            }

        checkpoint = await self._get_checkpoint(job_id)
        last_phase = checkpoint.get("last_phase")

        logger.info(f"Resuming import job {job_id} from phase: {last_phase}")

        # Re-run the import (it will skip already processed items)
        return await self.run_import(job_id, resume=True)

    async def cancel_job(self, job_id: str) -> bool:
        """Cancel an import job."""
        job = await self.get_job(job_id)
        if not job:
            return False

        if job.status in (ImportJobStatus.COMPLETED, ImportJobStatus.FAILED):
            return False

        await self.update_job(
            job_id,
            status=ImportJobStatus.CANCELLED,
            completed_at=utcnow(),
            progress_message="Import cancelled by user",
        )
        return True

    async def dry_run(self, job_id: str) -> dict | None:
        """
        Perform a dry-run simulation of the import.

        Detects:
        - Duplicate content (by wp_id or slug)
        - Potential conflicts
        - Warnings and errors

        Returns a detailed report without making any changes.
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        try:
            await self.update_job(
                job_id,
                status=ImportJobStatus.ANALYZING,
                phase=ImportJobPhase.ANALYZE,
                progress_message="Running dry-run simulation...",
            )

            # Get WXR data from source
            wxr_data = await self._fetch_source_data(job)
            if not wxr_data:
                raise ValueError("Failed to fetch source data")

            # Initialize result
            dry_run_result = {
                "success": True,
                "summary": {
                    "authors": {"total": 0, "new": 0, "duplicates": 0, "errors": []},
                    "categories": {"total": 0, "new": 0, "duplicates": 0, "errors": []},
                    "tags": {"total": 0, "new": 0, "duplicates": 0, "errors": []},
                    "media": {"total": 0, "new": 0, "duplicates": 0, "errors": []},
                    "posts": {"total": 0, "new": 0, "duplicates": 0, "errors": []},
                    "pages": {"total": 0, "new": 0, "duplicates": 0, "errors": []},
                },
                "duplicates": [],
                "warnings": [],
                "errors": [],
            }

            # Check authors
            await self._dry_run_check_authors(wxr_data, dry_run_result)

            # Check categories
            await self._dry_run_check_categories(wxr_data, dry_run_result)

            # Check tags
            await self._dry_run_check_tags(wxr_data, dry_run_result)

            # Check media
            config = job.config or {}
            if config.get("import_media", True):
                await self._dry_run_check_media(wxr_data, dry_run_result)

            # Check posts
            await self._dry_run_check_posts(wxr_data, dry_run_result, "post")

            # Check pages
            await self._dry_run_check_posts(wxr_data, dry_run_result, "page")

            # Store dry-run result in job
            await self.update_job(
                job_id,
                status=ImportJobStatus.PENDING,
                phase=ImportJobPhase.INIT,
                dry_run_result=dry_run_result,
                progress_message="Dry-run complete",
            )

            return dry_run_result

        except Exception as e:
            logger.exception(f"Dry-run failed for job {job_id}")
            return {
                "success": False,
                "error": str(e),
            }

    async def _dry_run_check_authors(
        self,
        wxr_data: WXRData,
        result: dict,
    ) -> None:
        """Check authors for duplicates."""
        summary = result["summary"]["authors"]
        summary["total"] = len(wxr_data.authors)

        for author in wxr_data.authors:
            existing = await self._find_by_wp_id("user", author.id)
            if existing:
                summary["duplicates"] += 1
                result["duplicates"].append({
                    "type": "author",
                    "wp_id": author.id,
                    "name": author.display_name or author.login,
                    "reason": "Already imported (wp_id match)",
                })
            else:
                # Check by email
                from ...models import EntityValue
                from sqlalchemy import select

                email_check = await self.db.execute(
                    select(Entity)
                    .join(EntityValue)
                    .where(
                        Entity.type == "user",
                        EntityValue.field_name == "email",
                        EntityValue.value_text == author.email,
                        Entity.deleted_at.is_(None),
                    )
                )
                if email_check.scalar_one_or_none():
                    summary["duplicates"] += 1
                    result["duplicates"].append({
                        "type": "author",
                        "wp_id": author.id,
                        "name": author.display_name or author.login,
                        "reason": f"Email already exists: {author.email}",
                    })
                else:
                    summary["new"] += 1

    async def _dry_run_check_categories(
        self,
        wxr_data: WXRData,
        result: dict,
    ) -> None:
        """Check categories for duplicates."""
        summary = result["summary"]["categories"]
        summary["total"] = len(wxr_data.categories)

        for cat in wxr_data.categories:
            existing = await self._find_by_wp_id("category", cat.id)
            if existing:
                summary["duplicates"] += 1
                result["duplicates"].append({
                    "type": "category",
                    "wp_id": cat.id,
                    "name": cat.name,
                    "reason": "Already imported (wp_id match)",
                })
            else:
                # Check by slug
                existing_slug = await self.entity_svc.find_by_slug("category", cat.slug)
                if existing_slug:
                    summary["duplicates"] += 1
                    result["duplicates"].append({
                        "type": "category",
                        "wp_id": cat.id,
                        "name": cat.name,
                        "reason": f"Slug already exists: {cat.slug}",
                    })
                else:
                    summary["new"] += 1

    async def _dry_run_check_tags(
        self,
        wxr_data: WXRData,
        result: dict,
    ) -> None:
        """Check tags for duplicates."""
        summary = result["summary"]["tags"]
        summary["total"] = len(wxr_data.tags)

        for tag in wxr_data.tags:
            existing = await self._find_by_wp_id("tag", tag.id)
            if existing:
                summary["duplicates"] += 1
                result["duplicates"].append({
                    "type": "tag",
                    "wp_id": tag.id,
                    "name": tag.name,
                    "reason": "Already imported (wp_id match)",
                })
            else:
                # Check by slug
                existing_slug = await self.entity_svc.find_by_slug("tag", tag.slug)
                if existing_slug:
                    summary["duplicates"] += 1
                    result["duplicates"].append({
                        "type": "tag",
                        "wp_id": tag.id,
                        "name": tag.name,
                        "reason": f"Slug already exists: {tag.slug}",
                    })
                else:
                    summary["new"] += 1

    async def _dry_run_check_media(
        self,
        wxr_data: WXRData,
        result: dict,
    ) -> None:
        """Check media for duplicates."""
        media_posts = [p for p in wxr_data.posts if p.post_type == "attachment"]
        summary = result["summary"]["media"]
        summary["total"] = len(media_posts)

        for media in media_posts:
            existing = await self._find_by_wp_id("media", media.id)
            if existing:
                summary["duplicates"] += 1
                result["duplicates"].append({
                    "type": "media",
                    "wp_id": media.id,
                    "name": media.title,
                    "reason": "Already imported (wp_id match)",
                })
            else:
                summary["new"] += 1

    async def _dry_run_check_posts(
        self,
        wxr_data: WXRData,
        result: dict,
        post_type: str,
    ) -> None:
        """Check posts/pages for duplicates."""
        posts = [p for p in wxr_data.posts if p.post_type == post_type]
        summary = result["summary"]["posts" if post_type == "post" else "pages"]
        summary["total"] = len(posts)

        for post in posts:
            existing = await self._find_by_wp_id(post_type, post.id)
            if existing:
                summary["duplicates"] += 1
                result["duplicates"].append({
                    "type": post_type,
                    "wp_id": post.id,
                    "title": post.title,
                    "reason": "Already imported (wp_id match)",
                })
            else:
                # Check by slug
                existing_slug = await self.entity_svc.find_by_slug(post_type, post.slug)
                if existing_slug:
                    summary["duplicates"] += 1
                    result["duplicates"].append({
                        "type": post_type,
                        "wp_id": post.id,
                        "title": post.title,
                        "reason": f"Slug already exists: {post.slug}",
                    })
                    result["warnings"].append({
                        "code": "SLUG_CONFLICT",
                        "message": f"{post_type.title()} '{post.title}' has conflicting slug: {post.slug}",
                        "wp_id": post.id,
                    })
                else:
                    summary["new"] += 1

    async def preview_import(
        self,
        job_id: str,
        limit: int = 3,
    ) -> dict | None:
        """
        Import a small number of items for preview.

        Imports up to `limit` posts/pages to let the user verify
        the import works correctly before doing the full import.

        Items are marked with is_preview=True for later confirmation or rollback.
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        try:
            await self.update_job(
                job_id,
                status=ImportJobStatus.IMPORTING,
                phase=ImportJobPhase.POSTS,
                progress_message="Creating preview import...",
            )

            # Get WXR data from source
            wxr_data = await self._fetch_source_data(job)
            if not wxr_data:
                raise ValueError("Failed to fetch source data")

            # Get posts for preview (prioritize published posts)
            all_posts = [p for p in wxr_data.posts if p.post_type == "post"]
            published_posts = [p for p in all_posts if p.status == "publish"]
            preview_posts = (published_posts or all_posts)[:limit]

            imported = []
            preview_ids = []

            for post in preview_posts:
                try:
                    existing = await self._find_by_wp_id("post", post.id)
                    if existing:
                        continue

                    post_data = {
                        "title": post.title,
                        "slug": f"preview-{post.slug}",  # Prefix to avoid conflicts
                        "content": post.content,
                        "excerpt": post.excerpt,
                        "status": "draft",  # Always draft for preview
                        "wp_id": post.id,
                        "wp_author_id": post.author_id,
                        "is_preview": True,  # Mark as preview
                        "created_at": post.created_at.isoformat(),
                        "updated_at": post.modified_at.isoformat(),
                    }

                    entity = await self.entity_svc.create("post", post_data)
                    entity_data = self.entity_svc.serialize(entity)
                    preview_ids.append(entity_data["id"])
                    imported.append({
                        "id": entity_data["id"],
                        "title": post.title,
                        "slug": entity_data.get("slug"),
                        "original_status": post.status,
                        "wp_id": post.id,
                        "content_preview": post.content[:500] if post.content else "",
                    })

                except Exception as e:
                    logger.warning(f"Failed to preview import post {post.id}: {e}")

            # Store preview IDs in job config for later confirmation/rollback
            config = job.config or {}
            config["preview_ids"] = preview_ids
            await self.update_job(
                job_id,
                status=ImportJobStatus.PENDING,
                phase=ImportJobPhase.INIT,
                config=config,
                progress_message="Preview complete",
            )

            return {
                "success": True,
                "preview_count": len(imported),
                "items": imported,
                "preview_ids": preview_ids,
            }

        except Exception as e:
            logger.exception(f"Preview failed for job {job_id}")
            return {
                "success": False,
                "error": str(e),
            }

    async def confirm_preview(self, job_id: str) -> dict | None:
        """Confirm preview items and update them to their original status."""
        job = await self.get_job(job_id)
        if not job:
            return None

        config = job.config or {}
        preview_ids = config.get("preview_ids", [])

        if not preview_ids:
            return {"success": False, "error": "No preview items to confirm"}

        confirmed = 0
        for entity_id in preview_ids:
            try:
                entity = await self.entity_svc.get("post", entity_id)
                if entity:
                    entity_data = self.entity_svc.serialize(entity)
                    # Remove preview prefix from slug
                    slug = entity_data.get("slug", "")
                    if slug.startswith("preview-"):
                        new_slug = slug[8:]  # Remove "preview-" prefix
                        await self.entity_svc.update("post", entity_id, {
                            "slug": new_slug,
                            "is_preview": False,
                        })
                    confirmed += 1
            except Exception as e:
                logger.warning(f"Failed to confirm preview {entity_id}: {e}")

        # Clear preview IDs
        config["preview_ids"] = []
        config["preview_confirmed"] = True
        await self.update_job(job_id, config=config)

        return {"success": True, "confirmed": confirmed}

    async def discard_preview(self, job_id: str) -> dict | None:
        """Discard preview items by deleting them."""
        job = await self.get_job(job_id)
        if not job:
            return None

        config = job.config or {}
        preview_ids = config.get("preview_ids", [])

        if not preview_ids:
            return {"success": False, "error": "No preview items to discard"}

        discarded = 0
        for entity_id in preview_ids:
            try:
                await self.entity_svc.delete("post", entity_id)
                discarded += 1
            except Exception as e:
                logger.warning(f"Failed to discard preview {entity_id}: {e}")

        # Clear preview IDs
        config["preview_ids"] = []
        config["preview_discarded"] = True
        await self.update_job(job_id, config=config)

        return {"success": True, "discarded": discarded}

    async def detect_diff(self, job_id: str) -> dict | None:
        """
        Detect differences between WordPress data and database.

        Returns a summary of new/updated/unchanged/deleted items.
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        try:
            await self.update_job(
                job_id,
                status=ImportJobStatus.ANALYZING,
                phase=ImportJobPhase.ANALYZE,
                progress_message="Detecting differences...",
            )

            # Get WXR data from source
            wxr_data = await self._fetch_source_data(job)
            if not wxr_data:
                raise ValueError("Failed to fetch source data")

            # Run diff detection
            from .diff_detector import DiffDetector

            detector = DiffDetector(self.db)
            diff_result = await detector.detect(wxr_data)

            # Store diff result in job config
            config = job.config or {}
            config["diff_result"] = diff_result.to_dict()
            await self.update_job(
                job_id,
                status=ImportJobStatus.PENDING,
                phase=ImportJobPhase.INIT,
                config=config,
                progress_message="Diff detection complete",
            )

            return {
                "success": True,
                **diff_result.to_dict(),
            }

        except Exception as e:
            logger.exception(f"Diff detection failed for job {job_id}")
            return {
                "success": False,
                "error": str(e),
            }

    async def import_diff(
        self,
        job_id: str,
        import_new: bool = True,
        import_updated: bool = True,
        delete_removed: bool = False,
    ) -> dict | None:
        """
        Import only the differences (new and updated items).

        Args:
            job_id: Import job ID
            import_new: Import new items
            import_updated: Import updated items (will update existing)
            delete_removed: Delete items that are no longer in WordPress
        """
        job = await self.get_job(job_id)
        if not job:
            return None

        config = job.config or {}
        diff_data = config.get("diff_result")

        if not diff_data:
            return {
                "success": False,
                "error": "No diff result found. Run detect_diff first.",
            }

        try:
            await self.update_job(
                job_id,
                status=ImportJobStatus.IMPORTING,
                phase=ImportJobPhase.POSTS,
                started_at=utcnow(),
                progress_message="Starting diff import...",
            )

            # Get WXR data again
            wxr_data = await self._fetch_source_data(job)
            if not wxr_data:
                raise ValueError("Failed to fetch source data")

            results = {
                "new_imported": 0,
                "updated": 0,
                "deleted": 0,
                "errors": [],
            }

            # Import new items
            if import_new:
                await self._import_new_from_diff(job_id, wxr_data, diff_data, results)

            # Update existing items
            if import_updated:
                await self._update_from_diff(job_id, wxr_data, diff_data, results)

            # Delete removed items
            if delete_removed:
                await self._delete_from_diff(job_id, diff_data, results)

            await self.update_job(
                job_id,
                status=ImportJobStatus.COMPLETED,
                phase=ImportJobPhase.COMPLETE,
                completed_at=utcnow(),
                progress_message="Diff import complete",
            )

            return {
                "success": True,
                **results,
            }

        except Exception as e:
            logger.exception(f"Diff import failed for job {job_id}")
            await self.update_job(
                job_id,
                status=ImportJobStatus.FAILED,
                completed_at=utcnow(),
                errors=[str(e)],
                progress_message=f"Diff import failed: {str(e)}",
            )
            return {
                "success": False,
                "error": str(e),
            }

    async def _import_new_from_diff(
        self,
        job_id: str,
        wxr_data: WXRData,
        diff_data: dict,
        results: dict,
    ) -> None:
        """Import new items from diff."""
        # Build lookup maps for WXR data
        posts_map = {p.id: p for p in wxr_data.posts if p.post_type == "post"}
        pages_map = {p.id: p for p in wxr_data.posts if p.post_type == "page"}
        media_map = {p.id: p for p in wxr_data.posts if p.post_type == "attachment"}
        cats_map = {c.id: c for c in wxr_data.categories}
        tags_map = {t.id: t for t in wxr_data.tags}
        authors_map = {a.id: a for a in wxr_data.authors}

        # Import new posts
        for item in diff_data.get("new", {}).get("posts", []):
            wp_id = item["wp_id"]
            post = posts_map.get(wp_id)
            if post:
                try:
                    # Sanitize content
                    content_result = self.sanitizer.sanitize(post.content or "")
                    excerpt_result = self.sanitizer.sanitize(post.excerpt or "")
                    if content_result.had_issues or excerpt_result.had_issues:
                        logger.warning(f"Sanitized content in new post {wp_id}")

                    await self.entity_svc.create("post", {
                        "title": post.title,
                        "slug": post.slug,
                        "content": content_result.content,
                        "excerpt": excerpt_result.content,
                        "status": WP_STATUS_MAP.get(post.status, "draft"),
                        "wp_id": post.id,
                        "wp_modified": post.modified_at.isoformat() if post.modified_at else None,
                    })
                    results["new_imported"] += 1
                except Exception as e:
                    results["errors"].append(f"Post {wp_id}: {e}")

        # Import new pages
        for item in diff_data.get("new", {}).get("pages", []):
            wp_id = item["wp_id"]
            page = pages_map.get(wp_id)
            if page:
                try:
                    # Sanitize content
                    content_result = self.sanitizer.sanitize(page.content or "")
                    if content_result.had_issues:
                        logger.warning(f"Sanitized content in new page {wp_id}")

                    await self.entity_svc.create("page", {
                        "title": page.title,
                        "slug": page.slug,
                        "content": content_result.content,
                        "status": WP_STATUS_MAP.get(page.status, "draft"),
                        "wp_id": page.id,
                        "wp_modified": page.modified_at.isoformat() if page.modified_at else None,
                    })
                    results["new_imported"] += 1
                except Exception as e:
                    results["errors"].append(f"Page {wp_id}: {e}")

        # Import new categories
        for item in diff_data.get("new", {}).get("categories", []):
            wp_id = item["wp_id"]
            cat = cats_map.get(wp_id)
            if cat:
                try:
                    await self.entity_svc.create("category", {
                        "name": cat.name,
                        "slug": cat.slug,
                        "wp_id": cat.id,
                    })
                    results["new_imported"] += 1
                except Exception as e:
                    results["errors"].append(f"Category {wp_id}: {e}")

        # Import new tags
        for item in diff_data.get("new", {}).get("tags", []):
            wp_id = item["wp_id"]
            tag = tags_map.get(wp_id)
            if tag:
                try:
                    await self.entity_svc.create("tag", {
                        "name": tag.name,
                        "slug": tag.slug,
                        "wp_id": tag.id,
                    })
                    results["new_imported"] += 1
                except Exception as e:
                    results["errors"].append(f"Tag {wp_id}: {e}")

        await self.update_job(
            job_id,
            progress_message=f"Imported {results['new_imported']} new items",
        )

    async def _update_from_diff(
        self,
        job_id: str,
        wxr_data: WXRData,
        diff_data: dict,
        results: dict,
    ) -> None:
        """Update existing items from diff."""
        posts_map = {p.id: p for p in wxr_data.posts if p.post_type == "post"}
        pages_map = {p.id: p for p in wxr_data.posts if p.post_type == "page"}
        cats_map = {c.id: c for c in wxr_data.categories}
        tags_map = {t.id: t for t in wxr_data.tags}

        # Update posts
        for item in diff_data.get("updated", {}).get("posts", []):
            wp_id = item["wp_id"]
            post = posts_map.get(wp_id)
            if post:
                try:
                    existing = await self._find_by_wp_id("post", wp_id)
                    if existing:
                        # Sanitize content
                        content_result = self.sanitizer.sanitize(post.content or "")
                        excerpt_result = self.sanitizer.sanitize(post.excerpt or "")
                        if content_result.had_issues or excerpt_result.had_issues:
                            logger.warning(f"Sanitized content in updated post {wp_id}")

                        await self.entity_svc.update("post", existing.id, {
                            "title": post.title,
                            "slug": post.slug,
                            "content": content_result.content,
                            "excerpt": excerpt_result.content,
                            "status": WP_STATUS_MAP.get(post.status, "draft"),
                            "wp_modified": post.modified_at.isoformat() if post.modified_at else None,
                        })
                        results["updated"] += 1
                except Exception as e:
                    results["errors"].append(f"Update post {wp_id}: {e}")

        # Update pages
        for item in diff_data.get("updated", {}).get("pages", []):
            wp_id = item["wp_id"]
            page = pages_map.get(wp_id)
            if page:
                try:
                    existing = await self._find_by_wp_id("page", wp_id)
                    if existing:
                        # Sanitize content
                        content_result = self.sanitizer.sanitize(page.content or "")
                        if content_result.had_issues:
                            logger.warning(f"Sanitized content in updated page {wp_id}")

                        await self.entity_svc.update("page", existing.id, {
                            "title": page.title,
                            "slug": page.slug,
                            "content": content_result.content,
                            "status": WP_STATUS_MAP.get(page.status, "draft"),
                            "wp_modified": page.modified_at.isoformat() if page.modified_at else None,
                        })
                        results["updated"] += 1
                except Exception as e:
                    results["errors"].append(f"Update page {wp_id}: {e}")

        # Update categories
        for item in diff_data.get("updated", {}).get("categories", []):
            wp_id = item["wp_id"]
            cat = cats_map.get(wp_id)
            if cat:
                try:
                    existing = await self._find_by_wp_id("category", wp_id)
                    if existing:
                        await self.entity_svc.update("category", existing.id, {
                            "name": cat.name,
                            "slug": cat.slug,
                        })
                        results["updated"] += 1
                except Exception as e:
                    results["errors"].append(f"Update category {wp_id}: {e}")

        # Update tags
        for item in diff_data.get("updated", {}).get("tags", []):
            wp_id = item["wp_id"]
            tag = tags_map.get(wp_id)
            if tag:
                try:
                    existing = await self._find_by_wp_id("tag", wp_id)
                    if existing:
                        await self.entity_svc.update("tag", existing.id, {
                            "name": tag.name,
                            "slug": tag.slug,
                        })
                        results["updated"] += 1
                except Exception as e:
                    results["errors"].append(f"Update tag {wp_id}: {e}")

        await self.update_job(
            job_id,
            progress_message=f"Updated {results['updated']} items",
        )

    async def _delete_from_diff(
        self,
        job_id: str,
        diff_data: dict,
        results: dict,
    ) -> None:
        """Delete items that were removed from WordPress."""
        entity_type_map = {
            "posts": "post",
            "pages": "page",
            "media": "media",
            "categories": "category",
            "tags": "tag",
            "authors": "user",
        }

        for diff_type, entity_type in entity_type_map.items():
            for item in diff_data.get("deleted", {}).get(diff_type, []):
                wp_id = item["wp_id"]
                try:
                    existing = await self._find_by_wp_id(entity_type, wp_id)
                    if existing:
                        await self.entity_svc.delete(entity_type, existing.id)
                        results["deleted"] += 1
                except Exception as e:
                    results["errors"].append(f"Delete {diff_type} {wp_id}: {e}")

        await self.update_job(
            job_id,
            progress_message=f"Deleted {results['deleted']} items",
        )

    async def fix_links(
        self,
        job_id: str,
        source_domain: str | None = None,
    ) -> dict:
        """
        Fix internal links in imported content.

        Rewrites WordPress URLs to Focomy URLs in post/page content.

        Args:
            job_id: Import job ID
            source_domain: Original WordPress domain for URL matching

        Returns:
            Dict with fix counts and details
        """
        await self.update_job(
            job_id,
            progress_message="Building URL map...",
        )

        # Build URL map
        builder = URLMapBuilder(self.db)
        url_map = await builder.build_map(source_domain)

        if not url_map:
            logger.info("No URL mappings found, skipping link fix")
            return {
                "success": True,
                "posts_fixed": 0,
                "pages_fixed": 0,
                "total_links_fixed": 0,
            }

        await self.update_job(
            job_id,
            progress_message=f"Fixing links with {len(url_map)} URL mappings...",
        )

        # Create link fixer
        fixer = InternalLinkFixer(url_map, source_domain)

        posts_fixed = 0
        pages_fixed = 0
        total_links_fixed = 0

        # Fix posts
        posts_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "post",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        posts = posts_result.scalars().unique().all()

        for post in posts:
            content_result = await self.db.execute(
                select(EntityValue).where(
                    EntityValue.entity_id == post.id,
                    EntityValue.field_name == "content",
                )
            )
            content_ev = content_result.scalar_one_or_none()

            if content_ev and content_ev.value_text:
                fix_result = fixer.fix_content(content_ev.value_text)
                if fix_result.had_fixes:
                    content_ev.value_text = fix_result.content
                    posts_fixed += 1
                    total_links_fixed += len(fix_result.fixes)

        # Fix pages
        pages_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "page",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        pages = pages_result.scalars().unique().all()

        for page in pages:
            content_result = await self.db.execute(
                select(EntityValue).where(
                    EntityValue.entity_id == page.id,
                    EntityValue.field_name == "content",
                )
            )
            content_ev = content_result.scalar_one_or_none()

            if content_ev and content_ev.value_text:
                fix_result = fixer.fix_content(content_ev.value_text)
                if fix_result.had_fixes:
                    content_ev.value_text = fix_result.content
                    pages_fixed += 1
                    total_links_fixed += len(fix_result.fixes)

        await self.db.commit()

        logger.info(
            f"Fixed links: {posts_fixed} posts, {pages_fixed} pages, "
            f"{total_links_fixed} total links"
        )

        await self.update_job(
            job_id,
            progress_message=f"Fixed {total_links_fixed} internal links",
        )

        return {
            "success": True,
            "posts_fixed": posts_fixed,
            "pages_fixed": pages_fixed,
            "total_links_fixed": total_links_fixed,
        }

    async def generate_redirects(
        self,
        job_id: str,
        source_url: str,
        config: dict | None = None,
    ) -> dict:
        """
        Generate URL redirects for imported content.

        Args:
            job_id: Import job ID
            source_url: Original WordPress site URL
            config: Optional configuration overrides

        Returns:
            Dict with redirect report data
        """
        await self.update_job(
            job_id,
            progress_message="Generating redirects...",
        )

        config = config or {}

        # Collect data from imported entities
        posts_data = []
        categories_data = []
        tags_data = []
        authors_data = []

        # Get posts
        posts_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "post",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        posts = posts_result.scalars().unique().all()

        for post in posts:
            slug = await self._get_entity_field(post.id, "slug")
            title = await self._get_entity_field(post.id, "title")
            if slug:
                posts_data.append({
                    "old_url": f"/{slug}",
                    "new_slug": slug,
                    "slug": slug,
                    "title": title or slug,
                    "post_type": "post",
                })

        # Get pages
        pages_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "page",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        pages = pages_result.scalars().unique().all()

        for page in pages:
            slug = await self._get_entity_field(page.id, "slug")
            title = await self._get_entity_field(page.id, "title")
            if slug:
                posts_data.append({
                    "old_url": f"/{slug}",
                    "new_slug": slug,
                    "slug": slug,
                    "title": title or slug,
                    "post_type": "page",
                })

        # Get categories
        cats_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "category",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        categories = cats_result.scalars().unique().all()

        for cat in categories:
            slug = await self._get_entity_field(cat.id, "slug")
            name = await self._get_entity_field(cat.id, "name")
            if slug:
                categories_data.append({
                    "slug": slug,
                    "old_slug": slug,
                    "new_slug": slug,
                    "name": name or slug,
                })

        # Get tags
        tags_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "tag",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        tags = tags_result.scalars().unique().all()

        for tag in tags:
            slug = await self._get_entity_field(tag.id, "slug")
            name = await self._get_entity_field(tag.id, "name")
            if slug:
                tags_data.append({
                    "slug": slug,
                    "old_slug": slug,
                    "new_slug": slug,
                    "name": name or slug,
                })

        # Get authors
        authors_result = await self.db.execute(
            select(Entity)
            .join(EntityValue, Entity.id == EntityValue.entity_id)
            .where(
                Entity.type == "user",
                EntityValue.field_name == "wp_id",
                Entity.deleted_at.is_(None),
            )
        )
        authors = authors_result.scalars().unique().all()

        for author in authors:
            login = await self._get_entity_field(author.id, "login")
            display_name = await self._get_entity_field(author.id, "display_name")
            if login:
                authors_data.append({
                    "login": login,
                    "new_slug": login,
                    "display_name": display_name or login,
                })

        # Generate redirects
        generator = RedirectGenerator(
            old_base_url=source_url,
            new_base_url=config.get("new_base_url", ""),
        )

        report = generator.generate_all(
            posts=posts_data,
            categories=categories_data,
            tags=tags_data,
            authors=authors_data,
            config=config,
        )

        logger.info(f"Generated {len(report.redirects)} redirects")

        await self.update_job(
            job_id,
            progress_message=f"Generated {len(report.redirects)} redirects",
        )

        return {
            "success": True,
            "redirect_count": len(report.redirects),
            "conflict_count": len(report.conflicts),
            "warnings": report.warnings,
            "redirects": [
                {
                    "from": r.from_path,
                    "to": r.to_path,
                    "status": r.status_code,
                    "regex": r.regex,
                    "comment": r.comment,
                }
                for r in report.redirects
            ],
            "conflicts": report.conflicts,
        }

    async def _get_entity_field(self, entity_id: str, field_name: str) -> str | None:
        """Get a string field value from an entity."""
        result = await self.db.execute(
            select(EntityValue).where(
                EntityValue.entity_id == entity_id,
                EntityValue.field_name == field_name,
            )
        )
        ev = result.scalar_one_or_none()
        if ev:
            return ev.value_string or ev.value_text
        return None



"""ThemeService - theme management and rendering."""

import json
import re
from datetime import datetime
from typing import Any

import yaml
from jinja2 import Environment, FileSystemLoader, select_autoescape
from markupsafe import escape

from ..config import settings
from .assets import get_asset_url, get_static_url, get_upload_url


class ThemeConfig:
    """Theme configuration."""

    def __init__(self, data: dict):
        self.name = data.get("name", "default")
        self.label = data.get("label", "Default Theme")
        self.version = data.get("version", "1.0.0")
        self.author = data.get("author", "")
        self.description = data.get("description", "")
        self.preview = data.get("preview", "")

        # CSS variables
        self.colors = data.get("colors", {})
        self.fonts = data.get("fonts", {})
        self.spacing = data.get("spacing", {})

        # Customization config
        self.config = data.get("config", {})

        # Widget areas and menu locations
        self.widget_areas = data.get("widget_areas", [])
        self.menu_locations = data.get("menu_locations", [])

        # Templates
        self.templates = data.get("templates", {})

        # Custom CSS
        self.custom_css = data.get("custom_css", "")


class ThemeService:
    """
    Theme management service.

    Features:
    - CSS variables from YAML config
    - Jinja2 template rendering
    - Template inheritance
    - Custom CSS injection
    """

    def __init__(self):
        self.themes_dir = settings.base_dir / "themes"
        self.themes_dir.mkdir(exist_ok=True)
        self._themes: dict[str, ThemeConfig] = {}
        self._current_theme: str = "default"
        self._env: Environment | None = None

    def _load_themes(self):
        """Load all theme configurations."""
        if self._themes:
            return

        for theme_dir in self.themes_dir.iterdir():
            if theme_dir.is_dir():
                config_path = theme_dir / "theme.yaml"
                if config_path.exists():
                    try:
                        with open(config_path, encoding="utf-8") as f:
                            data = yaml.safe_load(f)
                            if data:
                                theme = ThemeConfig(data)
                                self._themes[theme.name] = theme
                    except Exception as e:
                        print(f"Error loading theme {theme_dir}: {e}")

        # Ensure default theme exists
        if "default" not in self._themes:
            self._create_default_theme()

    def _create_default_theme(self):
        """Create default theme if not exists."""
        default_dir = self.themes_dir / "default"
        default_dir.mkdir(exist_ok=True)

        # Create theme.yaml
        config = {
            "name": "default",
            "label": "Default Theme",
            "version": "1.0.0",
            "colors": {
                "primary": "#2563eb",
                "primary-hover": "#1d4ed8",
                "background": "#ffffff",
                "surface": "#f8fafc",
                "text": "#1e293b",
                "text-muted": "#64748b",
                "border": "#e2e8f0",
                "success": "#22c55e",
                "error": "#ef4444",
                "warning": "#f59e0b",
            },
            "fonts": {
                "sans": "-apple-system, BlinkMacSystemFont, 'Segoe UI', Roboto, sans-serif",
                "serif": "Georgia, 'Times New Roman', serif",
                "mono": "ui-monospace, SFMono-Regular, Menlo, monospace",
            },
            "spacing": {
                "xs": "0.25rem",
                "sm": "0.5rem",
                "md": "1rem",
                "lg": "1.5rem",
                "xl": "2rem",
                "2xl": "3rem",
            },
        }

        with open(default_dir / "theme.yaml", "w", encoding="utf-8") as f:
            yaml.dump(config, f, allow_unicode=True, default_flow_style=False)

        # Create templates directory
        (default_dir / "templates").mkdir(exist_ok=True)

        # Create base template
        base_template = """<!DOCTYPE html>
<html lang="ja">
<head>
    <meta charset="UTF-8">
    <meta name="viewport" content="width=device-width, initial-scale=1.0">
    {% block meta %}{% endblock %}
    <title>{% block title %}{{ site_name }}{% endblock %}</title>
    <style>
        {{ theme_css }}
        {% block styles %}{% endblock %}
    </style>
</head>
<body>
    <header class="site-header">
        {% block header %}
        <div class="container">
            <a href="/" class="site-logo">{{ site_name }}</a>
            <nav class="site-nav">
                {% block nav %}{% endblock %}
            </nav>
        </div>
        {% endblock %}
    </header>

    <main class="site-main">
        {% block content %}{% endblock %}
    </main>

    <footer class="site-footer">
        {% block footer %}
        <div class="container">
            <p>&copy; {{ current_year }} {{ site_name }}</p>
        </div>
        {% endblock %}
    </footer>

    {% block scripts %}{% endblock %}
</body>
</html>
"""
        with open(default_dir / "templates" / "base.html", "w", encoding="utf-8") as f:
            f.write(base_template)

        # Create post template
        post_template = """{% extends "base.html" %}

{% block title %}{{ post.title }} - {{ site_name }}{% endblock %}

{% block meta %}
{{ seo_meta | safe }}
{% endblock %}

{% block content %}
<article class="post">
    <header class="post-header">
        <h1>{{ post.title }}</h1>
        <div class="post-meta">
            <time datetime="{{ post.created_at }}">{{ post.created_at[:10] }}</time>
        </div>
    </header>

    <div class="post-content">
        {{ post.body | render_blocks | safe }}
    </div>
</article>
{% endblock %}
"""
        with open(default_dir / "templates" / "post.html", "w", encoding="utf-8") as f:
            f.write(post_template)

        # Create index template
        index_template = """{% extends "base.html" %}

{% block title %}{{ site_name }}{% endblock %}

{% block content %}
<div class="posts-list">
    {% for post in posts %}
    <article class="post-card">
        <h2><a href="/{{ post.type }}/{{ post.slug }}">{{ post.title }}</a></h2>
        <p>{{ post.excerpt or (post.body | excerpt) }}</p>
        <time datetime="{{ post.created_at }}">{{ post.created_at[:10] }}</time>
    </article>
    {% else %}
    <p>No posts yet.</p>
    {% endfor %}
</div>
{% endblock %}
"""
        with open(default_dir / "templates" / "home.html", "w", encoding="utf-8") as f:
            f.write(index_template)

        # Create category template
        category_template = """{% extends "base.html" %}

{% block title %}{{ category.name }} - {{ site_name }}{% endblock %}

{% block content %}
<div class="category-page">
    <h1>{{ category.name }}</h1>
    {% if category.description %}
    <p class="category-description">{{ category.description }}</p>
    {% endif %}

    <div class="posts-list">
        {% for post in posts %}
        <article class="post-card">
            <h2><a href="/{{ post.type }}/{{ post.slug }}">{{ post.title }}</a></h2>
            <p>{{ post.excerpt or (post.body | excerpt) }}</p>
            <time datetime="{{ post.created_at }}">{{ post.created_at[:10] }}</time>
        </article>
        {% else %}
        <p>No posts in this category.</p>
        {% endfor %}
    </div>

    {% if total_pages > 1 %}
    <nav class="pagination">
        {% if page > 1 %}<a href="?page={{ page - 1 }}">&laquo; Prev</a>{% endif %}
        <span>Page {{ page }} of {{ total_pages }}</span>
        {% if page < total_pages %}<a href="?page={{ page + 1 }}">Next &raquo;</a>{% endif %}
    </nav>
    {% endif %}
</div>
{% endblock %}
"""
        with open(default_dir / "templates" / "category.html", "w", encoding="utf-8") as f:
            f.write(category_template)

        # Create archive template
        archive_template = """{% extends "base.html" %}

{% block title %}Archive: {{ year }}/{{ month }} - {{ site_name }}{% endblock %}

{% block content %}
<div class="archive-page">
    <h1>Archive: {{ year }}/{{ "%02d" | format(month) }}</h1>

    <div class="posts-list">
        {% for post in posts %}
        <article class="post-card">
            <h2><a href="/{{ post.type }}/{{ post.slug }}">{{ post.title }}</a></h2>
            <p>{{ post.excerpt or (post.body | excerpt) }}</p>
            <time datetime="{{ post.created_at }}">{{ post.created_at[:10] }}</time>
        </article>
        {% else %}
        <p>No posts in this period.</p>
        {% endfor %}
    </div>
</div>
{% endblock %}
"""
        with open(default_dir / "templates" / "archive.html", "w", encoding="utf-8") as f:
            f.write(archive_template)

        # Create search template
        search_template = """{% extends "base.html" %}

{% block title %}Search{% if query %}: {{ query }}{% endif %} - {{ site_name }}{% endblock %}

{% block content %}
<div class="search-page">
    <h1>Search</h1>

    <form action="/search" method="get" class="search-form">
        <input type="text" name="q" value="{{ query }}" placeholder="Search...">
        <button type="submit">Search</button>
    </form>

    {% if query %}
    <p class="search-results-count">{{ total }} result{% if total != 1 %}s{% endif %} for "{{ query }}"</p>

    <div class="posts-list">
        {% for post in posts %}
        <article class="post-card">
            <h2><a href="/{{ post.type }}/{{ post.slug }}">{{ post.title }}</a></h2>
            <p>{{ post.excerpt or (post.body | excerpt) }}</p>
            <time datetime="{{ post.created_at }}">{{ post.created_at[:10] }}</time>
        </article>
        {% else %}
        <p>No results found.</p>
        {% endfor %}
    </div>

    {% if total_pages > 1 %}
    <nav class="pagination">
        {% if page > 1 %}<a href="?q={{ query }}&page={{ page - 1 }}">&laquo; Prev</a>{% endif %}
        <span>Page {{ page }} of {{ total_pages }}</span>
        {% if page < total_pages %}<a href="?q={{ query }}&page={{ page + 1 }}">Next &raquo;</a>{% endif %}
    </nav>
    {% endif %}
    {% endif %}
</div>
{% endblock %}
"""
        with open(default_dir / "templates" / "search.html", "w", encoding="utf-8") as f:
            f.write(search_template)

        self._themes["default"] = ThemeConfig(config)

    def get_theme(self, name: str = None) -> ThemeConfig | None:
        """Get theme configuration."""
        self._load_themes()
        name = name or self._current_theme
        return self._themes.get(name)

    def get_all_themes(self) -> dict[str, ThemeConfig]:
        """Get all themes."""
        self._load_themes()
        return self._themes.copy()

    def set_current_theme(self, name: str):
        """Set current theme."""
        self._load_themes()
        if name in self._themes:
            self._current_theme = name
            self._env = None  # Reset environment

    def _minify_css(self, css: str) -> str:
        """Minify CSS by removing unnecessary whitespace and comments."""
        import re

        # Remove comments
        css = re.sub(r"/\*[\s\S]*?\*/", "", css)
        # Remove newlines and extra spaces
        css = re.sub(r"\s+", " ", css)
        # Remove spaces around special characters
        css = re.sub(r"\s*([{};:,>+~])\s*", r"\1", css)
        # Remove trailing semicolons before closing braces
        css = re.sub(r";\s*}", "}", css)
        return css.strip()

    def get_css_variables(self, theme_name: str = None, minify: bool = True) -> str:
        """Generate CSS variables from theme config."""
        theme = self.get_theme(theme_name)
        if not theme:
            return ""

        lines = [":root {"]

        # Colors
        for name, value in theme.colors.items():
            lines.append(f"  --color-{name}: {value};")

        # Fonts
        for name, value in theme.fonts.items():
            lines.append(f"  --font-{name}: {value};")

        # Spacing
        for name, value in theme.spacing.items():
            lines.append(f"  --space-{name}: {value};")

        lines.append("}")

        # Add base styles
        lines.append(
            """
* { box-sizing: border-box; margin: 0; padding: 0; }

body {
    font-family: var(--font-sans);
    background: var(--color-background);
    color: var(--color-text);
    line-height: 1.6;
}

.container {
    max-width: 1200px;
    margin: 0 auto;
    padding: 0 var(--space-lg);
}

.site-header {
    background: var(--color-surface);
    border-bottom: 1px solid var(--color-border);
    padding: var(--space-md) 0;
}

.site-header .container {
    display: flex;
    justify-content: space-between;
    align-items: center;
}

.site-logo {
    font-size: 1.5rem;
    font-weight: 700;
    color: var(--color-primary);
    text-decoration: none;
}

.site-nav a {
    color: var(--color-text);
    text-decoration: none;
    margin-left: var(--space-lg);
}

.site-nav a:hover {
    color: var(--color-primary);
}

.site-main {
    padding: var(--space-2xl) 0;
}

.site-footer {
    background: var(--color-surface);
    border-top: 1px solid var(--color-border);
    padding: var(--space-lg) 0;
    color: var(--color-text-muted);
    text-align: center;
}

/* Post styles */
.post {
    max-width: 800px;
    margin: 0 auto;
}

.post-header {
    margin-bottom: var(--space-xl);
}

.post-header h1 {
    font-size: 2.5rem;
    margin-bottom: var(--space-sm);
}

.post-meta {
    color: var(--color-text-muted);
}

.post-content {
    font-size: 1.125rem;
}

.post-content p {
    margin-bottom: var(--space-md);
}

.post-content h2, .post-content h3 {
    margin-top: var(--space-xl);
    margin-bottom: var(--space-md);
}

.post-content img {
    max-width: 100%;
    height: auto;
    border-radius: 0.5rem;
}

.post-content blockquote {
    border-left: 4px solid var(--color-primary);
    padding-left: var(--space-lg);
    margin: var(--space-lg) 0;
    font-style: italic;
    color: var(--color-text-muted);
}

.post-content pre {
    background: var(--color-surface);
    padding: var(--space-md);
    border-radius: 0.5rem;
    overflow-x: auto;
    font-family: var(--font-mono);
}

/* Post list styles */
.posts-list {
    max-width: 800px;
    margin: 0 auto;
}

.post-card {
    padding: var(--space-lg) 0;
    border-bottom: 1px solid var(--color-border);
}

.post-card h2 {
    font-size: 1.5rem;
    margin-bottom: var(--space-sm);
}

.post-card h2 a {
    color: var(--color-text);
    text-decoration: none;
}

.post-card h2 a:hover {
    color: var(--color-primary);
}

.post-card time {
    color: var(--color-text-muted);
    font-size: 0.875rem;
}

/* Checklist */
.checklist { list-style: none; padding: 0; }
.checklist-item { display: flex; align-items: center; gap: 0.5rem; padding: 0.25rem 0; }
.checklist-item input { width: 1.25rem; height: 1.25rem; }

/* Alert */
.alert { padding: 1rem 1.25rem; border-radius: 0.5rem; margin: 1rem 0; border-left: 4px solid; }
.alert--info { background: #eff6ff; border-color: #3b82f6; color: #1e40af; }
.alert--warning { background: #fffbeb; border-color: #f59e0b; color: #92400e; }
.alert--success { background: #f0fdf4; border-color: #22c55e; color: #166534; }
.alert--error { background: #fef2f2; border-color: #ef4444; color: #991b1b; }

/* Button */
.button-block { text-align: center; padding: 1rem 0; }
.btn { display: inline-block; padding: 0.75rem 2rem; border-radius: 0.5rem; text-decoration: none; font-weight: 600; }
.btn--primary { background: var(--color-primary); color: white; }
.btn--secondary { background: var(--color-surface); color: var(--color-text); border: 1px solid var(--color-border); }
.btn--outline { background: transparent; color: var(--color-primary); border: 2px solid var(--color-primary); }

/* Link card */
.linkcard { display: flex; gap: 1rem; border: 1px solid var(--color-border); border-radius: 0.5rem; padding: 1rem; text-decoration: none; color: inherit; margin: 1rem 0; }
.linkcard:hover { box-shadow: 0 4px 12px rgba(0,0,0,0.1); }
.linkcard__image { width: 120px; height: 80px; object-fit: cover; border-radius: 0.25rem; }
.linkcard__title { font-weight: 600; margin-bottom: 0.25rem; }
.linkcard__description { font-size: 0.875rem; color: var(--color-text-muted); }

/* Columns */
.columns { display: flex; gap: 1.5rem; margin: 1rem 0; }
.columns--2 .column { width: 50%; }
.columns--3 .column { width: 33.333%; }

/* Embed */
.embed { margin: 1.5rem 0; }
.embed iframe { border-radius: 0.5rem; }
.embed__caption { text-align: center; font-size: 0.875rem; color: var(--color-text-muted); margin-top: 0.5rem; }

/* Video embeds (YouTube, Vimeo, Twitch) - 16:9 aspect ratio */
.embed--youtube, .embed--vimeo, .embed--twitch { position: relative; padding-bottom: 56.25%; height: 0; }
.embed--youtube iframe, .embed--vimeo iframe, .embed--twitch iframe { position: absolute; top: 0; left: 0; width: 100%; height: 100%; }

/* Google Maps */
.embed--googleMaps iframe { width: 100%; height: 400px; border-radius: 0.5rem; }

/* Spotify */
.embed--spotify iframe { border-radius: 12px; }

/* SoundCloud */
.embed--soundcloud iframe { border-radius: 0.5rem; }

/* Social embeds */
.embed--twitter, .embed--instagram { max-width: 550px; margin-left: auto; margin-right: auto; }

/* Map */
.map { margin: 1.5rem 0; }
.map iframe { border-radius: 0.5rem; }
.map__caption { text-align: center; font-size: 0.875rem; color: var(--color-text-muted); margin-top: 0.5rem; }

/* Video */
.video { margin: 1.5rem 0; }
.video--youtube, .video--vimeo { position: relative; padding-bottom: 56.25%; height: 0; }
.video--youtube iframe, .video--vimeo iframe { position: absolute; top: 0; left: 0; width: 100%; height: 100%; border-radius: 0.5rem; }
.video--native video { width: 100%; border-radius: 0.5rem; }
.video__caption { text-align: center; font-size: 0.875rem; color: var(--color-text-muted); margin-top: 0.5rem; }
"""
        )

        # Add custom CSS
        if theme.custom_css:
            lines.append(theme.custom_css)

        css = "\n".join(lines)
        return self._minify_css(css) if minify else css

    def get_template_env(self, theme_name: str = None) -> Environment:
        """Get Jinja2 environment for theme."""
        theme = self.get_theme(theme_name)
        theme_dir = self.themes_dir / (theme.name if theme else "default") / "templates"

        env = Environment(
            loader=FileSystemLoader(str(theme_dir)),
            autoescape=select_autoescape(["html", "xml"]),
        )

        # Add custom filters
        env.filters["render_blocks"] = self._render_blocks
        env.filters["excerpt"] = self._excerpt
        env.filters["asset_url"] = get_asset_url
        env.filters["upload_url"] = get_upload_url
        env.filters["static_url"] = get_static_url
        env.filters["date"] = self._date_filter

        # Add global functions
        env.globals["asset_url"] = get_asset_url
        env.globals["upload_url"] = get_upload_url
        env.globals["static_url"] = get_static_url
        env.globals["now"] = datetime.now

        return env

    def _date_filter(self, value: Any, format: str = "%Y-%m-%d") -> str:
        """Format date/datetime value."""
        if value is None:
            return ""
        if isinstance(value, str):
            # Try to parse ISO format string
            try:
                from datetime import datetime
                value = datetime.fromisoformat(value.replace("Z", "+00:00"))
            except (ValueError, AttributeError):
                return value[:10] if len(value) >= 10 else value
        if hasattr(value, "strftime"):
            return value.strftime(format)
        return str(value)

    def _excerpt(self, content: Any, length: int = 200) -> str:
        """Extract plain text excerpt from content."""
        if not content:
            return ""

        text = ""

        # Handle dict (Editor.js blocks)
        if isinstance(content, dict) and "blocks" in content:
            texts = []
            for block in content.get("blocks", []):
                if block.get("type") in ("paragraph", "header"):
                    block_text = block.get("data", {}).get("text", "")
                    # Strip HTML tags
                    import re

                    block_text = re.sub(r"<[^>]+>", "", block_text)
                    texts.append(block_text)
            text = " ".join(texts)
        elif isinstance(content, str):
            # Try parsing as JSON
            try:
                data = json.loads(content)
                return self._excerpt(data, length)
            except (json.JSONDecodeError, TypeError):
                import re

                text = re.sub(r"<[^>]+>", "", content)
        else:
            text = str(content)

        # Truncate
        if len(text) <= length:
            return text
        return text[: length - 3].rsplit(" ", 1)[0] + "..."

    def _sanitize_url(self, url: str) -> str:
        """Sanitize URL to prevent XSS via javascript: or data: URLs."""
        if not url:
            return ""
        url = url.strip()
        # Allow only safe protocols
        if url.startswith(("http://", "https://", "/", "./", "../")):
            return escape(url)
        # Block javascript:, data:, vbscript:, etc.
        return ""

    def _sanitize_html(self, html: str, allow_tags: set = None) -> str:
        """Sanitize HTML allowing only specific tags."""
        if allow_tags is None:
            allow_tags = {"b", "i", "strong", "em", "a", "br", "u", "mark", "code"}

        # Simple regex-based sanitizer for inline content
        import re

        def replace_tag(match):
            tag = match.group(1).lower().split()[0]  # Get tag name without attributes
            if tag.lstrip("/") in allow_tags:
                return match.group(0)
            return escape(match.group(0))

        return re.sub(r"<(/?\w+[^>]*)>", replace_tag, html)

    def _render_blocks(self, content: Any) -> str:
        """Render Editor.js blocks to HTML."""
        if not content:
            return ""

        data = content

        # Handle string content (possibly JSON)
        if isinstance(content, str):
            try:
                data = json.loads(content)
                # Handle double-encoded JSON (legacy data)
                if isinstance(data, str):
                    data = json.loads(data.replace("\\!", "!"))
            except json.JSONDecodeError:
                return content

        if not isinstance(data, dict) or "blocks" not in data:
            return str(content)

        html_parts = []
        for block in data.get("blocks", []):
            block_type = block.get("type")
            block_data = block.get("data", {})

            if block_type == "paragraph":
                html_parts.append(f'<p>{block_data.get("text", "")}</p>')

            elif block_type == "header":
                level = block_data.get("level", 2)
                text = block_data.get("text", "")
                html_parts.append(f"<h{level}>{text}</h{level}>")

            elif block_type == "list":
                style = block_data.get("style", "unordered")
                tag = "ol" if style == "ordered" else "ul"
                items = block_data.get("items", [])
                items_html = "".join(f"<li>{item}</li>" for item in items)
                html_parts.append(f"<{tag}>{items_html}</{tag}>")

            elif block_type == "image":
                file_data = block_data.get("file", {})
                url = self._sanitize_url(file_data.get("url", ""))
                caption = escape(block_data.get("caption", ""))
                if url:
                    html_parts.append(
                        f'<figure><img src="{url}" alt="{caption}" loading="lazy"><figcaption>{caption}</figcaption></figure>'
                    )

            elif block_type == "quote":
                text = block_data.get("text", "")
                caption = block_data.get("caption", "")
                html_parts.append(f"<blockquote><p>{text}</p><cite>{caption}</cite></blockquote>")

            elif block_type == "code":
                code = escape(block_data.get("code", ""))
                html_parts.append(f"<pre><code>{code}</code></pre>")

            elif block_type == "delimiter":
                html_parts.append("<hr>")

            elif block_type == "table":
                content = block_data.get("content", [])
                rows = []
                for row in content:
                    cells = "".join(f"<td>{self._sanitize_html(cell)}</td>" for cell in row)
                    rows.append(f"<tr>{cells}</tr>")
                html_parts.append(f'<table>{"".join(rows)}</table>')

            elif block_type == "raw":
                # Sanitize raw HTML to prevent XSS
                from .sanitizer import sanitizer_service

                raw_html = block_data.get("html", "")
                html_parts.append(sanitizer_service.sanitize(raw_html))

            elif block_type == "embed":
                service = escape(block_data.get("service", ""))
                embed = self._sanitize_url(block_data.get("embed", ""))
                caption = escape(block_data.get("caption", ""))
                caption_html = (
                    f'<figcaption class="embed__caption">{caption}</figcaption>' if caption else ""
                )
                if not embed:
                    continue

                # サービスごとに最適なiframe設定
                if service == "youtube":
                    iframe = f'<iframe src="{embed}" width="100%" height="400" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>'
                elif service == "vimeo":
                    iframe = f'<iframe src="{embed}" width="100%" height="400" frameborder="0" allow="autoplay; fullscreen; picture-in-picture" allowfullscreen></iframe>'
                elif service == "googleMaps":
                    iframe = f'<iframe src="{embed}" width="100%" height="400" style="border:0" allowfullscreen loading="lazy" referrerpolicy="no-referrer-when-downgrade"></iframe>'
                elif service == "spotify":
                    iframe = f'<iframe src="{embed}" width="100%" height="352" frameborder="0" allow="autoplay; clipboard-write; encrypted-media; fullscreen; picture-in-picture" loading="lazy"></iframe>'
                elif service == "soundcloud":
                    iframe = f'<iframe src="{embed}" width="100%" height="166" scrolling="no" frameborder="no" allow="autoplay"></iframe>'
                elif service == "twitter":
                    iframe = (
                        f'<iframe src="{embed}" width="100%" height="500" frameborder="0"></iframe>'
                    )
                elif service == "instagram":
                    iframe = f'<iframe src="{embed}" width="100%" height="600" frameborder="0" scrolling="no" allowtransparency="true"></iframe>'
                elif service == "twitch":
                    iframe = f'<iframe src="{embed}" width="100%" height="400" frameborder="0" allowfullscreen></iframe>'
                else:
                    iframe = f'<iframe src="{embed}" width="100%" height="400" frameborder="0" allowfullscreen></iframe>'

                html_parts.append(
                    f'<figure class="embed embed--{service}">{iframe}{caption_html}</figure>'
                )

            elif block_type == "checklist":
                items = block_data.get("items", [])
                items_html = []
                for item in items:
                    checked = "checked" if item.get("checked") else ""
                    text = self._sanitize_html(item.get("text", ""))
                    items_html.append(
                        f'<li class="checklist-item"><input type="checkbox" {checked} disabled><span>{text}</span></li>'
                    )
                html_parts.append(f'<ul class="checklist">{"".join(items_html)}</ul>')

            elif block_type == "alert":
                alert_type = escape(block_data.get("type", "info"))
                message = self._sanitize_html(block_data.get("message", ""))
                html_parts.append(f'<div class="alert alert--{alert_type}">{message}</div>')

            elif block_type == "button":
                text = escape(block_data.get("text", "ボタン"))
                url = self._sanitize_url(block_data.get("url", "#")) or "#"
                style = escape(block_data.get("style", "primary"))
                html_parts.append(
                    f'<div class="button-block"><a href="{url}" class="btn btn--{style}">{text}</a></div>'
                )

            elif block_type == "linkCard":
                url = self._sanitize_url(block_data.get("url", ""))
                title = escape(block_data.get("title", ""))
                description = escape(block_data.get("description", ""))
                image = self._sanitize_url(block_data.get("image", ""))
                img_html = f'<img src="{image}" alt="" class="linkcard__image">' if image else ""
                if url:
                    html_parts.append(
                        f"""<a href="{url}" class="linkcard" target="_blank" rel="noopener">
                        {img_html}
                        <div class="linkcard__content">
                            <div class="linkcard__title">{title}</div>
                            <div class="linkcard__description">{description}</div>
                        </div>
                    </a>"""
                    )

            elif block_type == "columns":
                cols = block_data.get("content", [])
                cols_html = "".join(
                    f'<div class="column">{self._sanitize_html(col)}</div>' for col in cols
                )
                html_parts.append(f'<div class="columns columns--{len(cols)}">{cols_html}</div>')

            elif block_type == "map":
                src = self._sanitize_url(block_data.get("src", ""))
                caption = escape(block_data.get("caption", ""))
                caption_html = (
                    f'<figcaption class="map__caption">{caption}</figcaption>' if caption else ""
                )
                if src:
                    html_parts.append(
                        f"""<figure class="map">
                        <iframe src="{src}" width="100%" height="400" style="border:0" allowfullscreen loading="lazy" referrerpolicy="no-referrer-when-downgrade"></iframe>
                        {caption_html}
                    </figure>"""
                    )

            elif block_type == "video":
                url = block_data.get("url", "")
                caption = escape(block_data.get("caption", ""))
                caption_html = (
                    f'<figcaption class="video__caption">{caption}</figcaption>' if caption else ""
                )

                # YouTube - extract video ID safely
                yt_match = re.search(r"(?:youtube\.com/watch\?v=|youtu\.be/)([a-zA-Z0-9_-]+)", url)
                if yt_match:
                    video_id = escape(yt_match.group(1))
                    html_parts.append(
                        f"""<figure class="video video--youtube">
                        <iframe src="https://www.youtube.com/embed/{video_id}" width="100%" height="400" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe>
                        {caption_html}
                    </figure>"""
                    )
                    continue

                # Vimeo - extract video ID safely
                vimeo_match = re.search(r"vimeo\.com/(\d+)", url)
                if vimeo_match:
                    video_id = escape(vimeo_match.group(1))
                    html_parts.append(
                        f"""<figure class="video video--vimeo">
                        <iframe src="https://player.vimeo.com/video/{video_id}" width="100%" height="400" frameborder="0" allow="autoplay; fullscreen; picture-in-picture" allowfullscreen></iframe>
                        {caption_html}
                    </figure>"""
                    )
                    continue

                # Direct video (mp4, webm, ogg)
                safe_url = self._sanitize_url(url)
                if safe_url and re.search(r"\.(mp4|webm|ogg)(\?.*)?$", url, re.IGNORECASE):
                    html_parts.append(
                        f"""<figure class="video video--native">
                        <video src="{safe_url}" controls width="100%"></video>
                        {caption_html}
                    </figure>"""
                    )
                    continue

                # Generic iframe - only for safe URLs
                safe_url = self._sanitize_url(url)
                if safe_url:
                    html_parts.append(
                        f"""<figure class="video">
                        <iframe src="{safe_url}" width="100%" height="400" frameborder="0" allowfullscreen></iframe>
                        {caption_html}
                    </figure>"""
                    )

        return "\n".join(html_parts)

    def render(
        self,
        template_name: str,
        context: dict[str, Any],
        theme_name: str = None,
    ) -> str:
        """Render a template with context."""
        from datetime import datetime

        env = self.get_template_env(theme_name)
        template = env.get_template(template_name)

        # Add theme CSS to context
        context["theme_css"] = self.get_css_variables(theme_name)
        context["current_year"] = datetime.now().year
        context.setdefault("site_name", "Focomy")

        return template.render(**context)


# Singleton
theme_service = ThemeService()
